<?php
/*

   GOpdc: functions.php4 - Needed functions for GOpdc framework

   Author: Lars Scheiter <lars.scheiter@GONICUS.de>

Copyright (C) 2005 GONICUS GmbH

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA

 Contact information: GONICUS GmbH
                      Moehnestrasse 11-17
                      D-59755 Arnsberg
                      Germany
                      tel: ++49 2932 916 0
                      fax: ++49 2932 916 230
                      email: info@GONICUS.de
                      http://www.GONICUS.de
*/

/**
	Some helper functions for GOpdc
*/

/** 
 * Logfile generation function. Appends $text after timestamp.
 * @param $text Text to insert in Logfile
 */
function filelog($text){
	global $log_file;
	// sorry, couldnt resist ;)
	// TODO: Make it "real"
	system('echo '.date("Y/m/d H:i:s")." ".$text.' >> '.$log_file);
}

/**
 * Debug function. Uses global $log_channel to determine the log channel to use.
 * If global $debug_mode is set, the debug message will also be printed on stderr.
 * @param $text the debug message to print
 * @param $prio Priority of the debug message (Unused)
 */
function deBug($text,$prio="2"){
	global $log_channel,$debug_mode;
	if ($debug_mode) { $log_channel="stderr";}
	switch($log_channel) {
		case "file": {
			filelog($text);	
			break;
		}
		case "syslog": {
			syslog(LOG_DEBUG,"GOpdc: ".$text."\n");
			break;	
		}

		case "stderr": {
			fwrite(STDERR,"GOpdc: ".$text."\n");
			break;
		}
		
		case "both":{
			filelog($text);
			syslog(LOG_DEBUG,"GOpdc: ".$text."\n");
			break;
		}
		// Everything else will be silently dropped!
	}
	
}


/**
 * Calculates RID for given unix id
 * @param $id UNIX id
 * @param $ldap_con Existing LDAP connection handle
 * @param $what Calcutate RID for user or group (u = user (Default), g = group)
 * @return Calculated RID for given UNIX id
 */
function calcRid($id,$ldap_con,$what="u"){
	if ($what="g"){
		return(($id*2)+(getRidBase($ldap_con)))+1;
	} else {
		return(($id*2)+getRidBase($ldap_con));
	}
}

/*
	LDAP functions
*/


/**
 * Connect to $ldap_host
 * @return LDAP connection handle
 */
function ldapConnect(){
	global $ldap_host,$ldap_base,$ldap_binddn,$ldap_pass,
	$ldap_tls,$ldap_prot,$ldap_port;
	$my_connection = ldap_connect("$ldap_host", "$ldap_port");
	if (!$my_connection) {
		deBug("Failed connection to ".$ldap_host.", port ".$ldap_port);
		deBug(ldap_error($my_connection));
		exit(1);
	}
	if ($ldap_tls){
		if (!ldap_start_tls($my_connection)) {
			deBug("Failed to start TLS Session");
			deBug(ldap_error($my_connection));
			exit(1);
		}
	}
	if (!ldap_set_option($my_connection, LDAP_OPT_PROTOCOL_VERSION,$ldap_prot)){
		deBug("Failed to set protocol to ".$ldap_prot);
		deBug(ldap_error($my_connection));
		exit(1);
	}
	
	// Nearly every operation of GOpdc needs binded connection 
	if (!ldap_bind($my_connection,$ldap_binddn,$ldap_pass)){
		deBug("Cannot bind() to ldap");
		deBug(ldap_error($my_connection));
		exit(1);
	}
	return $my_connection;
}

/**
 * Disconnect from LDAP
 */
function ldapDisconnect($connection){
	if (!(ldap_close($connection))){
		// I hope nobody ever reaches this point!
		deBug("Cant close ldap connection!");
	}
}

/**
 * Simple search function
 * @param $connection Valid LDAP connection handle
 * @param $ldap_filter Search filter to use
 * @param $ldap_base Base of the LDAP tree
 * @param $ldap_attrs @see ldap_search
 * @param $ldap_scope Which scope to use (default is 'sub')
 * @return results
 */
function ldapSearch($connection,$ldap_filter,$ldap_base,$ldap_attrs=array("dn"),$ldap_scope="sub"){
	// TODO: Do we need scopes?

	// LDAP_SCOPE_SUBTREE 
	if ($ldap_scope == "sub") {
		$result=ldap_search($connection,$ldap_base,$ldap_filter,$ldap_attrs);
	} else {
	// LDAP_SCOPE_ONELEVEL
		$result=ldap_list($connection,$ldap_base,$ldap_filter,$ldap_attrs);
	}
	return ldap_get_entries($connection,$result);
}

// Yet useless function
function ldapGetAttributes($result,$attribute){
	array($fields);
	// TODO: Do i really need this function?
	// All attributes will be returned 
	$x=0;
	for ($y=0;$y<count($result);$y++){
		for ($i=0;$i<((count($result[$y][$attribute]))-1);$i++){
			$fields[$x]=$result[$y][$attribute][$i];
			$x++;
		}
	}
	return $fields;
}

// Count entries
function ldapCount($resultset){
	return (count($resultset))-1;
}

// Build up "our" base
function getBase($suffix=""){
	global $ldap_base;	
	if ($suffix=="") {
		$return=$ldap_base;
	} else {
		$return=$suffix.",".$ldap_base;
	}
	return($return);
}

// Check existance of user/machine
function checkUser($ldap_con,$username,$ldap_base){
	$results=ldapSearch($ldap_con,"(&(objectClass=posixAccount)(uid=".$username."))",$ldap_base,array("dn"));
	if (ldapCount($results)>0){
		return true;
	} else {
		return false;
	}
}

/** Get Domain sid
 * @param $ldap_con Valid LDAP connection handle
 * @return Domain SID
 */
function getDomainSid($ldap_con){
	global $GDATA,$ldap_base;
	$smb_domain=myOption("smb_domain");
	if (isset($GDATA["domsid"])){
		return $GDATA["domsid"];
	} else {
		$results=ldapSearch($ldap_con,"(&(objectClass=sambaDomain)
		(sambaDomainName=".$smb_domain."))",$ldap_base,array("sambaSID","sambaAlgorithmicRidBase"));
		// if we get more than one entry, theres something wicked
		if (ldapCount($results)>=2){
			deBug("getDomainSid got more than one entry for domain: ".$smb_domain);
			exit(1);
		} else {
			// Try to add entry
			if (ldapCount($results)<1){
				deBug("getDomainSid got no entries for domain: ".$smb_domain);
				exit(1);
			} else {
				$GDATA["domsid"]=$results[0]['sambasid'][0];
				$GDATA["ridbase"]=$results[0]['sambaalgorithmicridbase'][0];
				return($results[0]['sambasid'][0]);
			}
		}
	}
}

// Get Samba RIDBASE, all uids/gids will be higher or equal this number
function getRidBase($ldap_con){
	global $GDATA,$ldap_base;
	$smb_domain=myOption("smb_domain");
	if (isset($GDATA["ridbase"])){
		return $GDATA["ridbase"];
	} else {
		$results=ldapSearch($ldap_con,"(&(objectClass=sambaDomain)
		(sambaDomainName=".$smb_domain."))",$ldap_base,array("sambaAlgorithmicRidBase","sambaSID"));
		// if we get more than one entry, theres something wicked
		if (ldapCount($results)>=2){
			deBug("getRidBase got more than one entry for domain: ".$smb_domain);
			exit(1);
		} else {
			if (ldapCount($results)<1){
				// Essential
				debug("getRidBase got no entry for domain: ".$smb_domain);
				// TODO: Try to get it from samba config file
				//$GDATA["ridbase"] = getOption("rid_base");
				//return $GDATA["ridbase"];
				exit(1);
			} else {
				$GDATA["ridbase"]=$results[0][sambaalgorithmicridbase][0];
				$GDATA["domsid"]=$results[0][sambasid][0];
				return($results[0][sambaalgorithmicridbase][0]);
			}	
		}
	}
}


// Get primary group
function getPrimaryGroup($ldap_con,$grouptype="users",$fail=true,$domsid="enum"){
	global $GDATA,$ldap_base;
	if (isset($GDATA[$grouptype])){
		return $GDATA[$grouptype];
	} else {
		if ($domsid="enum"){
			$domsid=getDomainSid($ldap_con);
		}
		switch($grouptype) {
			// Domain users
			case "users": {
				$filter="(&(objectClass=posixGroup)(sambaSid=".$domsid."-513))";
				break;
			}
			// Domain admins
			case "admins": {
				$filter="(&(objectClass=posixGroup)(sambaSid=".$domsid."-512))";
				break;
			}
			// Domain guests
			case "guests": {
				$filter="(&(objectClass=posixGroup)(sambaSid=".$domsid."-514))";
				break;
			}
			// Domain machines
			case "machines": {
				$filter="(&(objectClass=posixGroup)(sambaSid=".$domsid."-516))";
				break;
			}
		}
		$result=ldapSearch($ldap_con,$filter,$ldap_base,array("gidNumber"));
		// Check for empty result
		if (!$result["count"] == "0") {
			$return=$result[0]["gidnumber"][0];
			if (count($return)<1){
				deBug("Cant find primary group id for ".$grouptype);
				if ($fail){
					exit(0);
				} else {
					return(false);
				}
			} else {
        			if (count($return)>1){
            				deBug("Found more than one group for ".$grouptype);
	        	    		exit(1);
        			} else {
					// Only cache found entries
					$GDATA[$grouptype]=$return;
	    				return($return);
				}
			}
		} else {
			return(false);
		}
        }
}


// Add samba attributes to group object
function ldapSmbGrpAttr($ldap_con,$group_name){
	global $ldap_basei,$add_id_map;
	$results=ldapSearch($ldap_con,"(&(cn=".$group_name.")(objectClass=posixGroup))",$ldap_base,array("dn",
	"gidNumber"));
	if (ldapCount($results)<1){
		deBug("ldapSmbGrpAttr: Group ".$group_name." not found");
		exit(1);
	} else {
		if (ldapCount($results)>1){
			deBug("More than one group with name ".$group_name." found");
			exit(1);
		} else {
			$my_sid=getDomainSid($ldap_con)."-".calcRid($results[0]["gidnumber"][0],$ldap_con,"g");
			$ldapmodadd=array("objectClass" => "sambaGroupMapping","sambaGroupType" => "2", 
			"sambaSID" => $my_sid);
			// TODO: Add id mapping
			if (ldap_mod_add($ldap_con,$results[0]["dn"],$ldapmodadd)){
				deBug("Added samba attributes to group ".$group_name);
			} else {
				deBug("LDAP problem adding samba attributes to group ".$group_name);
				deBug(ldap_error($ldap_con));
				exit(1);
			}

			
		}
	}
}


// Get the next free uid/gid
function getNextID($ldap_con,$what="uid"){
	global $GDATA,$ldap_base;
	$ux_uid=myOption("ux_uid");
	$ux_gid=myOption("ux_gid");
	if ($what == "gid"){
		if (isset($GDATA["gid"])){
			return $GDATA["gid"];
		} else {
			$lastid=$ux_gid;
			$myarray[0]=$ux_gid;
			$results=ldapSearch($ldap_con,"(objectClass=posixGroup)",$ldap_base,array("gidNumber"));
			for ($i=0;$i<(count($results)-1);$i++){
				if ($results[$i]['gidnumber'][0]>=$ux_gid){
					$myarray[$i]=$results[$i]['gidnumber'][0];
				}
			}
		}
	} else {
		if (isset($GDATA["uid"])){
			return $GDATA["uid"];
		} else {
			$lastid=$ux_uid;
			$myarray[0]=$ux_uid;
			$results=ldapSearch($ldap_con,"(objectClass=posixAccount)",$ldap_base,array("uidNumber"));
			for ($i=0;$i<(count($results)-1);$i++){
				if ($results[$i]['uidnumber'][0]>=$ux_uid){
					$myarray[$i]=$results[$i]['uidnumber'][0];
				}
			}
		}
	}
	sort($myarray);
	for ($i=0;$i<count($myarray);$i++){
		// The first is always false
		if (($lastid)!=$myarray[$i]) {
			break;
		}
		$lastid=($lastid+1);
	}
	
	// Cache data
	$GDATA[$what]=$lastid;
	return $lastid;
}

// Add group to LDAP
function ldapGroupAdd($ldap_con,$groupname,$base,$gid,$description="Added by GOpdc"){
	$ldapadd["objectclass"][0]="posixGroup";
	$ldapadd["description"]=$description;
	$ldapadd["gidNumber"]=$gid;
	$ldapadd["cn"]=$groupname;
	return(ldap_add($ldap_con,"cn=".$groupname.",".$base,$ldapadd));
}


// Add user to LDAP 
function ldapUserAdd($ldap_con,$uid,$gid,$username,$cn,$base,$dnmode="uid",$gecos="Added by GOpdc",$shell="/bin/true",$homedir="/tmp"){
	$ldapadd["objectclass"][0]="posixAccount";
	$ldapadd["objectclass"][1]="person";
	$ldapadd["objectclass"][2]="organizationalPerson";
	$ldapadd["objectclass"][3]="inetOrgPerson";
	$ldapadd["objectclass"][4]="shadowAccount";
  if (myOption("add_gosa_acct")){
    $ldapadd["objectclass"][5]="gosaAccount";
  }
	$ldapadd["shadowLastChange"]="12394";
	$ldapadd["uidNumber"]=$uid;
	$ldapadd["gidNumber"]=$gid;
	$ldapadd["loginShell"]=$shell;
	$ldapadd["gecos"]=$gecos;
	$ldapadd["shadowMin"]="0";
	$ldapadd["shadowWarning"]="0";
	$ldapadd["shadowInactive"]="0";
	$ldapadd["uid"]=$username;
	$ldapadd["cn"]=$cn;
	$ldapadd["sn"]=$cn;
	$ldapadd["givenName"]=$username;
	$ldapadd["homeDirectory"]=$homedir;
	// Sometimes we dont have enough information :(
	if ($dnmode=="cn"){
		return(ldap_add($ldap_con,"cn=".$cn.",".$base,$ldapadd));
	} else {
		return(ldap_add($ldap_con,"uid=".$username.",".$base,$ldapadd));
	}
}

// Delete user from LDAP
function ldapUserDel($ldap_con,$user_name){
	global $ldap_base;
	$result=ldapSearch($ldap_con,"(&(uid=".$user_name.")(objectClass=posixAccount))",$ldap_base,array("dn"));
	if (ldapCount($result)>1){
		deBug("ldapUserDel found too many entries for user ".$user_name);
		exit(1);
	} else {
		if (ldapCount($result)<1){
			// we do not exit here, maybe samba has "ldap delete dn" set!
			deBug("ldapUserDel found no user ".$user_name);
		} else {
			// actually delete the user
			if(ldap_delete($ldap_con,$result[0]["dn"])){
				deBug("user ".$user_name." deleted");
			} else {
				deBug("Cannot delete user ".$user_name." LDAP error");
				deBug(ldap_error($ldap_con));
				exit(1);
			}
		}
	}
			
}

// Delete users primary group
function ldapPrimGroupDel($ldap_con,$user_name){
	global $ldap_base;
	$result=ldapSearch($ldap_con,"(&(cn=".$user_name.")(objectClass=posixGroup))",$ldap_base,array("dn"));
	if (ldapCount($result)>1){
		// TODO: Mmm, should we exit(1) here?
		deBug("ldapPrimGroupDel found too many entries for user ".$user_name);
	} else {
		if (ldapCount($result)<1){
			// This may be okay, dont exit
			deBug("Primary group for user ".$user_name." not found");
		} else {
			if(ldap_delete($ldap_con,$result[0]["dn"])){
				deBug("primary group of user ".$user_name." deleted");
			} else {
				// If the entry exists it must be deleteable!
				deBug("Cannot delete primary group of user ".$user_name." LDAP error");
				deBug(ldap_error($ldap_con));
				exit(1);
			}
		}
	}
}

/**
 * Delete user from groups
 * @param $ldap_con Existing LDAP-Connection Handle
 * @param $user_name Name of the user which should be deleted from groups
 */
function ldapDelUsrGrp($ldap_con,$user_name){
	global $ldap_base;
	$ldapdelete["memberuid"]=$user_name;
	$results=ldapSearch($ldap_con,"(&(memberUid=".$user_name.")(objectClass=posixGroup))",$ldap_base,array("dn"));
	if (ldapCount($results)>0){
		for ($i=0;$i<ldapCount($results);$i++){
			if (ldap_mod_del($ldap_con,$results[$i]["dn"],$ldapdelete)){
				deBug("user ".$user_name." deleted from ".$results[$i]["dn"]);
			} else {
				// This is fatal!
				deBug("Error deleting user ".$user_name." from ".$results[$i]["dn"]);
				exit(1);
			}
		}
	}
}

/**
 * Delete group
 * @param $ldap_con Existing LDAP-Connection Handle
 * @param $groupname The group that should be deleted
 */
function ldapDeleteGroup($ldap_con,$groupname){
	global $ldap_base;
	$results=ldapSearch($ldap_con,"(&(cn=".$groupname.")(objectClass=posixGroup))",$ldap_base,array("dn"));
	if (ldapCount($results)<1){
		deBug("ldapDeleteGroup: group ".$groupname." not found");
		exit(1);
	} else {
		if(ldapCount($results)>1){
			deBug("ldapDeleteGroup: found more than one entry for group ".$groupname);
			exit(1);
		} else {
			if(ldap_delete($ldap_con,$results[0]["dn"])){
				deBug("group ".$groupname." deleted");
			} else {
				deBug("LDAP error while deleting ".$results[0]["dn"]);
				deBug(ldap_error($ldap_con));
				exit(1);
			}
		}
	}
}

/**
 * Add user to group named by gid
 * @param $ldap_con Existing LDAP-Connection Handle
 * @param $user_name The user that should be added to given group
 * @param $gid UNIX id of the the group the given user should be added to
 */
function add_usr_numgrp($ldap_con,$user_name,$gid){
	global $ldap_base;
	$results=ldapSearch($ldap_con,"(&(gidNumber=".$gid.")(objectClass=posixGroup))",$ldap_base,array("dn"));
	if (ldapCount($results)<1){
		deBug("Group with gidnumber ".$gid." not found");
		exit(1);
	} else {
		if (ldapCount($results)>1){
			deBug("add_usr_numgrp: found more than one group with gidnumber ".$gid);
			exit(1);
		} else {
			$ldapmodadd["memberuid"]=$user_name;
			if (ldap_mod_add($ldap_con,$results[0]["dn"],$ldapmodadd)){
				deBug("User ".$user_name." added to group with id ".$gid);
			} else {
				deBug("LDAP problem adding user ".$user_name." to group with gid ".$gid);
				deBug(ldap_error($ldap_con));
				exit(1);
			}
		}
	}
}

/**
 * Add samba attributes to object
 * @param $ldap_con Existing LDAP-Connection Handle
 * @param $user_name Add samba attributes to user_name
 * @param $base Unused
 * @param $user_id Unused
 */
function ldapSmbUsrMod($ldap_con,$user_name,$base=false,$user_id=false){
	global $ldap_base;
	/** TODO: finish this! */
	if (!$base){
		$searchbase=$ldap_base;
	} else {
		$searchbase=$base;
	}
	$results=ldapSearch($ldap_con,"(&(uid=$user_name)(objectClass=posixAccount))",
	$searchbase,array("dn","uidNumber","objectClass"));
	if (ldapCount($results)>1){
		deBug("Found more than one entry for user ".$user_name);
		exit(1);
	}
	if (!$user_id){
		$user_id=$results["uidnumber"][0];
	}
	$user_dn=$results[0]["dn"];
   
	// Get existing entries
	$ldapmodify["objectclass"]=$results[0]["objectclass"];
	
	// We don't want the "count" entry
	if(isset($ldapmodify["objectclass"]["count"])) {
		unset($ldapmodify["objectclass"]["count"]);
	}
		
	// What do we need to add
	array_push($ldapmodify["objectclass"],"sambaSamAccount");
	$ldapmodify["sambaSID"]=getDomainSid($ldap_con)."-".$user_id;

	//deBug("Yo: ".$user_dn." YoYo: ".$user_id);
   
	if ($user_dn != "") {
		return(ldap_modify($ldap_con, $user_dn, $ldapmodify));
	} else {
		deBug("No dn. Cannot add samba attributes");
		exit(1);
	}
}


/*
	Helper functions
*/

/**
 * Execute command
 * @param $command will be executed
 */
function postExec($command){
	/** TODO: Add some checks around this! */
	/*if (!$command==""){
		if(!system($command)){
			deBug("Failed to execute command ".$command);
		}
	}*/
	system($command);
}


/*
	Base functions
*/

/**
 * Add user
 * @param $user_name Name of new user
 */
function add_user($user_name){
	global $ldap_suf_user,$ldap_suf_group,$ldap_base,
	$post_exec_user;
	$realbase=getBase($ldap_suf_user);
	$ldap_con=ldapConnect();
	if (myOption("create_users")){
		// We assume here that the users location may be somewhere under $ldap_base
		if (checkUser($ldap_con,$user_name,$ldap_base)){
			deBug("User ".$user_name." is already listed under ".$ldap_base);
			// Exiting with error, this is not correct 
			exit(1);
		}
		// Create a primary group for user?
		if (myOption("add_prim_group")) {
			$group_id=getNextID($ldap_con,"gid");
			$group_base=getBase($ldap_suf_group);
			if(!ldapGroupAdd($ldap_con,$user_name,$group_base,$group_id)) {
				deBug("LDAP Problem adding group for new user ".$user_name." to base ".$group_base);
				deBug(ldap_error($ldap_con));
				exit(1);
			} else {
				deBug("Group ".$user_name." added to base ".$group_base);
				if (myOption("add_prim_smb")){ ldapSmbGrpAttr($ldap_con,$user_name);}
			}
		} else {
			$group_id=getPrimaryGroup($ldap_con,"users");
		}
		$user_id=getNextID($ldap_con,"uid");
		if(ldapUserAdd($ldap_con,$user_id,$group_id,
		$user_name,$user_name,$realbase,"uid","Windows user added by GOpdc")) {
			deBug("Added user ".$user_name." to base ".$realbase);
		} else {
			deBug("LDAP error adding user ".$user_name." to base ".$realbase);
			deBug(ldap_error($ldap_con));
			exit(1);
		}

		// User has a new primary group and alos needs to be member of "Domain Users"
		if (myOption("user_prim_domain") && myOption("add_prim_group")){
			add_usr_numgrp($ldap_con,$user_name,getPrimaryGroup($ldap_con,"users"));
		}
        
	        // Also add Samba attributes?
        	if (myOption("add_samba_attr")) {
	            ldapSmbUsrMod($ldap_con,$user_name,$realbase,$user_id);
		}

		// Execute command
		postExec($post_exec_user);	

	} else {
		deBug("GOpdc is not configured to add users");
	}
	
	// Close connection
	ldapDisconnect($ldap_con);
	// Exit cleanly
	exit(0);
}


/**
 * Delete user
 * @param $user_name Name of user to be deleted
 */
function del_user($user_name){
	global $post_exec_user;
	if (myOption("delete_users")){
		$ldap_con=ldapConnect();
		ldapUserDel($ldap_con,$user_name);
		ldapPrimGroupDel($ldap_con,$user_name);
		ldapDelUsrGrp($ldap_con,$user_name);
		
		// Execute command
		postExec($post_exec_user);	
		
		// Close connection
		ldapDisconnect($ldap_con);
	} else {
		deBug("GOpdc is not configured to delete users");
	}

   	// Exit cleanly 
	exit(0);
}

/**
 * Delete group
 * @param $groupname Name of group that should be deleted
 */
function del_group($groupname){
	global $post_exec_grup;
	if (myOption("delete_groups")){
		$ldap_con=ldapConnect();

		ldapDeleteGroup($ldap_con,$groupname);

		// Execute command
		postExec($post_exec_grup);

		// Disconnect
		ldapDisconnect($ldap_con);
	} else {
		deBug("GOpdc is not configured to delete groups");
	}

	// exit cleanly
	exit(0);
}

/** 
 * Add group
 * @param $groupname Name of new group
 */
function add_group($groupname){
	global $post_exec_grup, $ldap_suf_group;
	$ldap_con=ldapConnect();
	$realbase=getBase($ldap_suf_group);
	if (myOption("create_groups")){
		if(ldapGroupAdd($ldap_con,$groupname,$realbase,getNextID($ldap_con,"gid"),$description="Added by GOpdc")){
			deBug("group ".$groupname." added");	
		} else {
			deBug("LDAP error adding group ".$groupname);
			deBug(ldap_error($ldap_con));
			exit(1);
		}

		// Execute command
		postExec($post_exec_grup);

	} else {
		deBug("GOpdc is not configured to add groups");
	}

	// Disconnect
	ldapDisconnect($ldap_con);
	// exit
	exit(0);
}

/**
 * Delete user from group
 * @param $username Name of user that should be deleted from group
 * @param $groupname Name of group the user should be deleted from
 */
function del_user2group($username,$groupname){
	GLOBAL $ldap_base,$post_exec_grup;
	if (myOption("mod_groups")){	
		$userfound=false;
		$ldap_con=ldapConnect();
		$results=ldapSearch($ldap_con,"(&(cn=".$groupname.")(objectClass=posixGroup))",
		$ldap_base,array("dn","memberUid"));
		if (ldapCount($results)<1){
			debug("group ".$groupname." not found");
			exit(1);
		} else {
			if (ldapCount($results)>1){
				deBug("More than one group with name ".$groupname." found");
				exit(1);
			} else {
				for ($i=0;$i<ldapCount($results[0]["memberuid"]);$i++){
					if($results[0]["memberuid"][$i]==$username){
						$userfound=true;
						$ldapmoddel["memberuid"]=$username;
						if (ldap_mod_del($ldap_con,$results[0]["dn"],$ldapmoddel)){
							deBug("user ".$username." removed from group ".$groupname);
						} else {
							deBug("LDAP error while removing user ".$username." from
							group ".$groupname);
							deBug(ldap_error($ldap_con));
							exit(1);
						}
					}
				}
				if(!$userfound){
					deBug("user ".$username." is not in group ".$groupname);
					exit(1);
				}
			}
		}
		
		// Execute command
		postExec($post_exec_grup);

		// Disconnect
		ldapDisconnect($ldap_con);
	} else {
		deBug("GOpdc is not configured to modify groups");
	}

	// exit cleanly
	exit(0);
}

/**
 * Add user to group
 * @param $username Name of user that should be added to group
 * @param $groupname Name of group the user should be added to
 */
function add_user2group($username,$groupname){
	global $ldap_base,$post_exec_grup;
	if (myOption("mod_groups")){	
		$ldap_con=ldapConnect();
		if (checkUser($ldap_con,$username,$ldap_base)){
			$results=ldapSearch($ldap_con,"(&(cn=".$groupname.")(objectClass=posixGroup))",
			$ldap_base,array("dn","memberUid"));
			if (ldapCount($results)<1){
				deBug("group ".$groupname." not found");
				exit(1);
			} else {
				if (ldapCount($resuls)>1){
					deBug("More than one group with name ".$groupname." found");
					exit(1);
				} else {
					for ($i=0;$i<ldapCount($results[0]["memberuid"]);$i++){
						if($results[0]["memberuid"][$i]==$username){
							deBug("user ".$username." is already a member of group 
							".$groupname);
							exit(1);
						}
					}
					$ldapmodadd["memberuid"]=$username;
					if (ldap_mod_add($ldap_con,$results[0]["dn"],$ldapmodadd)){
						deBug("User ".$username." added to group ".$groupname);
					} else {
						deBug("LDAP problem adding user ".$username." to group ".$groupname);
						deBug(ldap_error($ldap_con));
						exit(1);
					}
				}
			}
		} else {
			deBug("user ".$username." not found");
			exit(1);
		}

		// Execute command
		postExec($post_exec_grup);

		// Disconnect
		ldapDisconnect($ldap_con);
	} else {
		deBug("GOpdc is not configured to modify group entries");
	}

	// Exit cleanly
	exit(0);
}


/**
 * Function to add a new machine
 * @param $machine_name Name of the new machine account
 */
function add_machine($machine_name){
	global $ldap_suf_mach,$ldap_base,$post_exec_mach;
	$realbase=getBase($ldap_suf_mach);
	if (myOption("create_machines")){
		$ldap_con=ldapConnect();
		// We assume here that the machine can be located somewhere under $ldap_base
		if (checkUser($ldap_con,$machine_name,$ldap_base)){
			deBug("Machine ".$machine_name." is already listed under ".$ldap_base);
			// Exiting cleanly since this may be correct!
			exit(0);
		}
		if(getOption('mach_own_grp')){
			$my_dist_group=getPrimaryGroup($ldap_con,"machines");
		} else {
			$my_dist_group=getPrimaryGroup($ldap_con,"users");
		}
		if(ldapUserAdd($ldap_con,getNextID($ldap_con,"uid"),$my_dist_group,
		$machine_name,"Windows ".$machine_name,$realbase,"Windows Workstation ".$machine_name,"cn")){
			deBug("Added machine ".$machine_name." to base ".$realbase);
		} else {
			deBug("Problem adding machine ".$machine_name." to base ".$realbase);
			exit(1);
		}
		
		// Execute command
		postExec($post_exec_mach);	

		// Close connection
		ldapDisconnect($ldap_con);
	} else {
		deBug("GOpdc is not configured to add machines.");
	}
	
	// Exit cleanly
	exit(0);
}

/**
 * Add samba attributes to user object (Not functional)
 * @param $user_name Name of the user the attributes should be added to
 */
function add_smbuser($user_name){
	/** TODO: add :) */
}

/**
 * Add samba attributes to group object
 * @param $group_name Name of the group the attributes should be added to
 */
function add_smbgroup($group_name){
	global $ldap_base,$ldap_suf_mach,$post_exec_mach;
	if (myOption("mod_groups")){
		$ldap_con=ldapConnect();

		ldapSmbGrpAttr($ldap_con,$group_name);

		// Execute command
		postExec($post_exec_mach);	

		// Close connection
		ldapDisconnect($ldap_con);
	} else {
		deBug("GOpdc is not configured to modify groups");
	}
	exit(0);
}

/**
 * Simple testing function
 */
function test_config(){
	global $smb_domsid,$ldap_base;
	deBug("Connecting to LDAP Server");
	$ldap_con=ldapConnect();
	deBug("Samba SID for ".myOption("smb_domain")." is ".getDomainSid($ldap_con).", Ridbase ".getRidBase($ldap_con));
	deBug("Domain admins have unix gid ".getPrimaryGroup($ldap_con,"admins"));
	deBug("Domain users have unix gid ".getPrimaryGroup($ldap_con,"users"));
	deBug("Domain guests have unix gid ".getPrimaryGroup($ldap_con,"guests"));
	deBug("Domain machines have unix gid ".getPrimaryGroup($ldap_con,"machines"));
	deBug("Next unix uid will be: ".getNextID($ldap_con,"uid").", gid will be ".getNextID($ldap_con,"gid"));
	deBug("Next Windows User RID will be: ".getDomainSid($ldap_con)."-".
	calcRid(getNextID($ldap_con,"uid"),$ldap_con,"u"));
	deBug("Next Windows Group RID will be: ".getDomainSid($ldap_con)."-".
	calcRid(getNextID($ldap_con,"gid"),$ldap_con,"g"));
	if (isset($smb_domsid)) { deBug("Domain SID is obsolete and can savely be removed from config!"); }
	if (myOption("add_prim_group")) {
		deBug("GOpdc will add a primary group for every new user");
	} else {
		deBug("Newly added users will have their primary group set to domain users");
	}
	
	// We are done, exit here
	deBug("Closing Connection to LDAP Server");
	ldapDisconnect($ldap_con);
}

/**
 * Simple workaround script to avoid problems with special characters.
 * ATM: Replaces german umlauts with their latin equivalents.
 */
function validate_name($string) {
  // be UTF-8 clean
  //$string= utf8_decode(utf8_encode($string));

  // lowercase all characters
  //$string= strtolower($string);

  // Replacement array
  //$array=array(
  //  "\xc3\xa4" => "ae",
  //  "\xc3\xb6" => "oe",
  //  "\xc3\xbc" => "ue",
  //  "\xc3\x9f" => "ss"
  //);
  
  // Replace special characters (ATM german umlauts and "sharp s")
  //$string= strtr($string,$array);
  return $string;
}

// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:
?>
