#!/usr/bin/php4 -q
<?php
/*

GOpdc: gopdc.php4 - Main part of GOpdc 

Authors: Lars Scheiter <lars.scheiter@GONICUS.de>
Jan Wenzel    <jan.wenzel@GONICUS.de>

Copyright (C) 2005 GONICUS GmbH

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Contact information: GONICUS GmbH
Moehnestrasse 11-17
D-59755 Arnsberg
Germany
tel: ++49 2932 916 0
fax: ++49 2932 916 230
email: info@GONICUS.de
http://www.GONICUS.de
*/



/** 
 * @brief GOpdc script (runable)
 */

// Read configuration
require_once("/usr/share/gopdc/config.inc");

/** The users home directory */
$ux_homedir=       getOption('ux_homedir');

/** 
 * Adds a group to every user and sets its primary group to this group. 
 * Beware that this will break "usrmgr"!
 */
$add_prim_group=   getOption('add_prim_group');

/**
 * Adds Samba Attributes to users primary group.
 * This is only valid with $add_prim_group=true
 */
$add_prim_smb=     getOption('add_prim_smb');

/**
 * Adds new users to group "Domain Users".
 * Only valid if add_prim_group=true
 */
$user_prim_domain= getOption('user_prim_domain');

/** Adds Samba attributes to every new object. This will break "usrmgr" */
$add_samba_attr=   getOption('add_samba_attr');

/** 
 * Verify that RID is not already used. Makes really only sense if 
 * add_samba_attr=true and/or add_prim_smb=true. 
 * You may enable this if you vampired (net rpc vampire) 
 * your domain and have not enough IDs left.
 */
$verify_rid=       getOption('verify_rid');

/** Adds sambaIdMapEntry ObjectClass to samba Accounts. Same prerequisites as $verify_rid */
$add_id_map=       getOption('add_id_map');

/** Hostname of the LDAP-Server */
$ldap_host=        getOption('ldap_host');

/** Base of the LDAP-Tree */
$ldap_base=        getOption('ldap_base');

/** BindDN to use for connection */
$ldap_binddn=      getOption('ldap_binddn');

/** Related password for BindDN */
$ldap_pass=        getOption('ldap_pass');

/** Whether to use TLS to connect to LDAP Server */
$ldap_tls=         getOption('ldap_tls');

/** The port to connect to */
$ldap_port=        getOption('ldap_port');

/** LDAP protocol version to use (Valid values are 1, 2, 3) */
$ldap_prot=        getOption('ldap_prot');

/** Suffix for users. Will be prepended to base, presets to base if not set */
$ldap_suf_user=    getOption('ldap_suf_user');

/** Suffix for groups. Will be prepended to base, presets to base if not set */
$ldap_suf_group=   getOption('ldap_suf_group');

/** Suffix for machines. Will be prepended to base, presets to base if not set */
$ldap_suf_mach=    getOption('ldap_suf_mach');

/** Should a POSIX Account be created */
$create_users=     getOption('create_users');

/** Should the POSIX Account be deleted */
$delete_users=     getOption('delete_users');

/** Should a machine Account be created */
$create_machines=  getOption('create_machines');

/** Should a POSIX Group be created */
$create_groups=    getOption('create_groups');

/** Should the POSIX Group be deleted */
$delete_groups=    getOption('delete_groups');

/** */
$mod_groups=       getOption('mod_groups');

/** Command to be executed after user has been created */
$post_exec_user=   getOption('post_exec_user');

/** Command to be executed after machine account has been created */
$post_exec_mach=   getOption('post_exec_mach');

/** Command to be execeuted after group has been created */
$post_exec_grup=   getOption('post_exec_grup');

/** Program Path to be included */
$prog_path=        getOption('prog_path');

/** 
 * Which log channel to use. 
 * Possible values are stderr, file, syslog, or both (file+syslog) 
 */
$log_channel=      getOption('log_channel');

/** Which log file to use if file logging is active */
$log_file=         getOption('log_file');

/** Just doing error reporting in our own ;) */
error_reporting(getOption('errorreporting'));

//! Load functions
require_once ($prog_path."functions.php4");

// Global defaults, may be useful
$default_ar=array(
	"ux_uid" 		=> "5001",
	"ux_gid" 		=> "5001",
  "ldap_port" => "389",
  "ldap_prot" => "3",
  "ldap_tls"  => false,
	"ux_homedir"		=> "/home",
	"create_users"		=> true,
	"delete_users"		=> true,
	"create_machines"	=> true,
	"create_groups"		=> true,
	"delete_groups"		=> true,
	"mod_groups"		=> true,
	"add_prim_group"	=> false,
	"add_samba_attr"	=> false,
	"add_prim_smb"		=> false,
	"user_prim_domain"	=> false,
	"verify_rid"		=> true,
  "add_gosa_acct" => true
);

/**
 * Prints out brief information about possible options
 * (With console switch -usage or any other invalid option)
 * Applies only to command line use.
 */
function usAge(){
	global $debug_mode,$log_channel;
	// TODO: Add more
	// Usage makes only sense if anybody can read it ;)
	if ($debug_mode || $log_channel=="stderr") {
		$usage="** This script is not ment to be started at the command line! **\n\n"
		."However the following options are currently available:\n"
		."gopdc.php [-debug] option parameter\n"
		."option must be one of:\n"
		."Common options:\n"
		." -adduser [username]		Add user\n"
		." -addgroup [groupname]		Add group\n"
		." -addmachine [machinename]	Add machine\n"
		." -deluser [username]		Delete user\n"
		." -delgroup [groupname]		Delete group\n"
		." -usr2grp [username] [groupname] Add user to group\n"
		." -usrFgrp [username] [groupname] Del user from group\n"
		."\nNoncommon:\n"
		." -addsmbusr [username]		Add samba attributes to user\n"
		." -addsmbgrp [groupname]		Add samba attributes to group\n"
		."** Please note that these options may break those entries! **\n"
		."\nmultiple options are ignored, may be mixed only with debugging option\n\n"
		."Testing/debugging options:\n"
		." -usage			The text you are reading right now\n"
		." -test			Simple but usefull config and function test\n"
		." -debug			Debugging mode, switches debugging to stdout\n"
		."			Maybe usefull in interactive mode\n"
		."** This script is not ment to be started at the command line! **\n";
		echo $usage;
	} else {
		deBug("gopdc was called with false/none options");
	}
}

// Parse commandline
if ($argc<2) { usAge(); }
for($i=1;$i<$argc;$i++){
	switch($argv[$i]) {
		/** @cond */
		case "-usage":
			$debug_mode=true;
			usAge();
			exit;
			break;

		case "-debug":
			$debug_mode=true;
			break;

		case "-test":
			// is always "interactive"
			$debug_mode=true;
			test_config();
			break;

		case "-adduser":
			$user_name=$argv[($i+1)];
      $user_name=validate_name($user_name);
			if (!$user_name) { usAge(); }
			add_user($user_name);
			break;

		case "-addgroup":
			$group_name=$argv[($i+1)];
      $group_name=validate_name($group_name);
			if (!$group_name) { usAge(); }
			add_group($group_name);
			break;

		case "-deluser":
			$user_name=$argv[($i+1)];
      $user_name=validate_name($user_name);
			if (!$user_name) { usAge(); }
			del_user($user_name);
			break;

		case "-delgroup":
			$group_name=$argv[($i+1)];
      $group_name=validate_name($group_name);
			if (!$group_name) { usAge(); }
			del_group($group_name);
			break;

		case "-usr2grp":
			$user_name=$argv[($i+1)];
			$group_name=$argv[($i+2)];
      $user_name=validate_name($user_name);
      $group_name=validate_name($group_name);
			if ((!$group_name) || (!$user_name)) { usAge(); }
			add_user2group($user_name,$group_name);
			break;

		case "-usrFgrp":
			$user_name=$argv[($i+1)];
			$group_name=$argv[($i+2)];
      $user_name=validate_name($user_name);
      $group_name=validate_name($group_name);
			if ((!$group_name) || (!$user_name)) { usAge(); }
			del_user2group($user_name,$group_name);
			break;

		case "-addmachine":
			$machine_name=$argv[($i+1)];
      $machine_name=validate_name($machine_name);
			if (!$machine_name) { usAge(); }
			add_machine($machine_name);
			break;

		case "-addsmbgrp":
			$group_name=$argv[($i+1)];
      $group_name=validate_name($group_name);
			if (!$group_name) { usAge(); }
			add_smbgroup($group_name);
			break;

		case "-addsmbusr":
			$user_name=$argv[($i+1)];
      $user_name=validate_name($user_name);
			if (!$user_name) { usAge(); }
			add_smbuser($user_name);
			break;

		default:
			usAge();
			exit(1);
		/** @endcond */
	}
}
// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:
?>
